/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.dasrecovery;

import org.glassfish.api.admin.config.Named;
import org.glassfish.api.admin.RestRedirect;
import org.glassfish.api.admin.RestRedirects; 

import org.jvnet.hk2.config.ConfigBeanProxy;
import org.jvnet.hk2.config.Attribute;
import org.jvnet.hk2.config.Configured;

import javax.validation.Payload;
import javax.validation.constraints.Min;
import javax.validation.constraints.Pattern;


/**
 * Configuration for DAS automatic backup and recovery.
 * 
 *
 * @author Chris Kasso
 *
 */


@Configured
@RestRedirects({
 @RestRedirect(opType = RestRedirect.OpType.POST, commandName = "create-backup-config"),
 @RestRedirect(opType = RestRedirect.OpType.DELETE, commandName = "delete-backup-config")
})   

public interface BackupConfig extends ConfigBeanProxy, Named, Payload {

    @Pattern(regexp="[\\p{L}\\p{N}_][\\p{L}\\p{N}\\-_\\.;]*",
             message="{backup.config.name.pattern}", 
             payload=BackupConfig.class)
    @Override
    public String getName();


    @Attribute (defaultValue="true", dataType=Boolean.class)
    String getAutoBackupEnabled();

    /**
     * Sets the value of the auto-backup-enabled attribute.
     * If it is true automatic backup of the domain is enabled.
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setAutoBackupEnabled(String value);

    @Attribute (defaultValue="daily") /* Default: everyday at 12AM */
    String getSchedule();

    /**
     * Sets the value of the schedule attribute.  This should be the name
     * of a schedule (see the create-schedule command).
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setSchedule(String value);

    @Attribute
    String getBackupDir();

    /**
     * Sets the value of the directory attribute.
     * This attribute controls which directory the backups will be stored to.
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setBackupDir(String value);

    @Attribute(defaultValue = "25", dataType=Integer.class)
    @Min(value=0)
    String getRecycleLimit(); 
 
    /**
     * Sets the value of the recycle-limit attribute.
     * This attribute controls how many backups for the domain will be
     * maintained before older backups are deleted.  A value of 0 indicates
     * an unlimited pool of backups will be maintained.
     *   
     * @param value allowed object is
     *              {@link String } 
     */  
    void setRecycleLimit(String value);

    @Attribute (defaultValue="false", dataType=Boolean.class)
    String getActiveBackupEnabled();

    /**
     * Sets the value of the active-backup-enabled attribute.
     * If it is true an automatic backup will occur without first
     * quiesceing the DAS.
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setActiveBackupEnabled(String value);

    @Attribute (defaultValue="false", dataType=Boolean.class)
    String getConfigOnly();

    /**
     * Sets the value of the config-only attribute.
     * If it is true the DAS configuration will be backed up.  Applications,
     * logs will not be backed up.  If it is false (default) the entire
     * domain will be backed up.
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setConfigOnly(String value);

    @Attribute
    String getTimeOfLastBackupAttempt();

    /**
     * Private attribute.
     *
     * Sets the value of the time-of-last-backup-attempt attribute.
     * A private property that records when the last automatic backup was
     * attempted regardless of whether it was successful.  
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setTimeOfLastBackupAttempt(String value);

    @Attribute
    String getTimeOfLastBackupSuccess();

    /**
     * Private attribute.
     *
     * Sets the value of the time-of-last-backup-success attribute.
     * A private property that records when the last automatic backup was
     * successfully attempted.  
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setTimeOfLastBackupSuccess(String value);
}
