/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.dasrecovery;

import org.glassfish.api.admin.config.ConfigExtension;

import org.jvnet.hk2.config.Attribute;
import org.jvnet.hk2.config.Element;
import org.jvnet.hk2.config.Configured;
import org.jvnet.hk2.config.DuckTyped;

import java.util.List;


/**
 * BackupConfigs configuration.  This defines a backup-configs element which 
 * holds a list of backup-config elements.
 *
 * @author Chris Kasso
 *
 */


@Configured
public interface BackupConfigs extends ConfigExtension {

    //XXX: @Element() results in a "backup-configs" element being created.
    @Element("backup-config")
    public List<BackupConfig> getBackupConfigs();

    /**
     * Return the backup config with the given name, or null if no such 
     * config exists.
     *   
     * @param   name    the name of the backup config
     * @return          the BackupConfig object, or null if no such config
     */  
    @DuckTyped
    public BackupConfig getBackupConfigByName(String name);

    class Duck {
        public static BackupConfig getBackupConfigByName(
					      BackupConfigs backupConfigs,
					      String name) {
            if (name == null || backupConfigs == null) {
                return null;
            }
            for (BackupConfig backupConfig : backupConfigs.getBackupConfigs()) {
                if (backupConfig.getName().equals(name)) {
                    return backupConfig;
                }
            }    
            return null;
        }
    }    
}
