/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.dasrecovery;


import org.glassfish.api.I18n;
import org.glassfish.api.ActionReport;
import org.glassfish.api.ActionReport.ExitCode;
import org.glassfish.api.admin.*;
import org.glassfish.api.Param;

import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.universal.i18n.LocalStringsImpl;
import com.sun.enterprise.util.StringUtils;

import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.component.PerLookup;
import org.jvnet.hk2.component.Habitat;
import org.jvnet.hk2.config.ConfigSupport;
import org.jvnet.hk2.config.SingleConfigCode;
import org.jvnet.hk2.config.TransactionFailure;


/**
 * Remote asadmin command: delete-backup-config
 * 
 * Purpose: Allows the invoker to delete a backup configuration element.
 * 
 * 
 * @author Chris Kasso
 *
 */

@Service(name="delete-backup-config")
@ExecuteOn(RuntimeType.DAS)  // Only appropriate for the DAS
@Scoped(PerLookup.class)
@I18n("backup.delete.command")
public class DeleteBackupConfig implements AdminCommand {

    @Inject(name=ServerEnvironment.DEFAULT_INSTANCE_NAME)
    Config config; 

    @Inject
    private Habitat habitat;

    @I18n("backup.delete.name")
    @Param(primary = true, optional = false)
    private String name;

    private static final LocalStringsImpl strings =
            new LocalStringsImpl(DeleteBackupConfig.class);


    public void execute(AdminCommandContext context) {

        ActionReport report = context.getActionReport();

	BackupConfigs backupConfigs =
				 config.getExtensionByType(BackupConfigs.class);

        /**
         * Check to see if BackupConfigs exists in this Config. 
         */
        if (backupConfigs == null) { 
	    report.setMessage(strings.get("backup.delete.notfound"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        /**
         * Find a specific backup config.
         */
        if (!StringUtils.ok(name)) {
	    report.setMessage(strings.get("backup.delete.badname"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        /**
         * Find a specific backup config.
         */
        // The named backup config can not be found.
        if (backupConfigs.getBackupConfigByName(name) == null) {
            report.setMessage(strings.get("backup.delete.notfound", name));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

	try {
            ConfigSupport.apply(new SingleConfigCode<BackupConfigs>() {

                public Object run(BackupConfigs param) throws 
							 TransactionFailure {
                        BackupConfig backupConfig = 
					      param.getBackupConfigByName(name);
                        param.getBackupConfigs().remove(backupConfig);
                        return param;
                }
            }, backupConfigs); 
	} catch(TransactionFailure e) {
	    report.setMessage(strings.get("backup.delete.failure", name));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            report.setFailureCause(e);
            return;
        }

        report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
        return;
    }
}
