/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.dasrecovery;

import org.glassfish.api.admin.config.Named;
import org.glassfish.api.admin.RestRedirect;
import org.glassfish.api.admin.RestRedirects; 

import org.jvnet.hk2.config.ConfigBeanProxy;

import org.jvnet.hk2.config.Attribute;
import org.jvnet.hk2.config.Configured;

import javax.validation.Payload;
import javax.validation.constraints.Pattern;


/**
 * Schedule configuration.  This defines a schedule which can be used
 * by reoccurring operations such as automatic domain backup.
 *
 * @author Chris Kasso
 *
 */

@Configured
@RestRedirects({
 @RestRedirect(opType = RestRedirect.OpType.POST, commandName = "create-schedule"),
 @RestRedirect(opType = RestRedirect.OpType.DELETE, commandName = "delete-schedule")
}) 
public interface Schedule extends ConfigBeanProxy, Named, Payload {

 
    @Pattern(regexp="[\\p{L}\\p{N}_][\\p{L}\\p{N}\\-_\\.;]*",
             message="{schedule.create.name.pattern}",
             payload=Schedule.class)
    @Override
    public String getName();

    @Attribute (defaultValue="0")
    String getSecond();

    /**
     * Sets the value of the second attribute.
     * One or more seconds within a minute.
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setSecond(String value);


    @Attribute (defaultValue="0")
    String getMinute();

    /**
     * Sets the value of the minute attribute.
     * One or more minutes within a minute.
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setMinute(String value);


    @Attribute (defaultValue="0")
    String getHour();

    /**
     * Sets the value of the hour attribute.
     * One or more hours within a day.
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setHour(String value);


    @Attribute (defaultValue="*")
    String getDayOfMonth();

    /**
     * Sets the value of the day-of-month attribute.
     * One or more days within a month.
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setDayOfMonth(String value);


    @Attribute (defaultValue="*")
    String getMonth();

    /**
     * Sets the value of the month attribute.
     * One or more months within a year.
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setMonth(String value);


    @Attribute (defaultValue="*")
    String getDayOfWeek();

    /**
     * Sets the value of the day-of-week attribute.
     * One or more days within a week.
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setDayOfWeek(String value);


    @Attribute (defaultValue="*")
    String getYear();

    /**
     * Sets the value of the year attribute.
     * A particular calendar year
     *
     * @param value allowed object is
     *              {@link String }
     */
    void setYear(String value);

}
