/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */
package com.sun.glassfish.dasrecovery;

import org.glassfish.api.Param;
import org.glassfish.api.ActionReport;
import org.glassfish.api.admin.AdminCommand;
import org.glassfish.api.admin.AdminCommandContext;
import org.glassfish.api.admin.ServerEnvironment;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.component.Habitat;
import org.jvnet.hk2.component.PerLookup;
import com.sun.enterprise.util.StringUtils;
import com.sun.enterprise.config.serverbeans.Config;
import com.sun.enterprise.universal.i18n.LocalStringsImpl;

/**
 * This is a hidden command which will call the BackupManager and  will back up now .
 * The BackupManager invokes the local backup command backup-domain to backup
 * on a suspended DAS and resume the DAS based on the value of the active-backup-enabled attribute.
 * If active-backup-enabled is true an automatic backup will occur without first
 * quiesceing the DAS.
 * Once the backup is done it will resume the DAS
 * @author Bhakti Mehta
 */
@Service(name = "_start-backup")
@Scoped(PerLookup.class)
public class StartBackup implements AdminCommand {

    @Inject
    Habitat habitat;

    @Param (primary=true)
    String configName;

    @Inject(name= ServerEnvironment.DEFAULT_INSTANCE_NAME)
    Config config;

    private static final LocalStringsImpl strings =
                new LocalStringsImpl(StartBackup.class);


    public void execute(AdminCommandContext context) {
        ActionReport report = context.getActionReport();
        BackupConfigs backupConfigs =
                config.getExtensionByType(BackupConfigs.class);

        /**
         * Check if any backupConfigs exist in this Config.
         */
        if (backupConfigs == null) {
            // If the user provided a name then it's an error if it is missing.
            if (StringUtils.ok(configName)) {
                report.setMessage(strings.get("backup.list.notfound", configName));
                report.setActionExitCode(ActionReport.ExitCode.FAILURE);
                return;
            }

            // No backupConfigs exist so just return.
            report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
            return;
        }
        BackupConfig backupConfig =
                backupConfigs.getBackupConfigByName(configName);

        // The named backupConfig can not be found.
        if (backupConfig == null) {
            report.setMessage(
                    strings.get("backup.list.notfound", configName));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }
        //The hidden command will call the BackupManager to execute
        //the suspend-domain (based on value of active-backup-enabled)
        //backup-domain
        //resume-domain

        BackupManager backupManager = habitat.getByContract(BackupManager.class);
        backupManager.backup(backupConfig,context);

    }
}
