/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 */

package com.sun.glassfish.quiesce;

import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.glassfish.api.I18n;
import org.glassfish.api.admin.config.ConfigExtension;
import org.glassfish.api.ActionReport;
import org.glassfish.api.ActionReport.ExitCode;
import org.glassfish.api.admin.*;
import org.glassfish.api.Param;

import com.sun.enterprise.universal.i18n.LocalStringsImpl;
import com.sun.enterprise.util.StringUtils;

import org.jvnet.hk2.annotations.Service;
import org.jvnet.hk2.annotations.Inject;
import org.jvnet.hk2.annotations.Scoped;
import org.jvnet.hk2.component.PerLookup;

/**
 * Remote asadmin command: resume-domain
 * 
 * Purpose: Allows the invoker to un-quiesce the DAS.
 * 
 * @author Chris Kasso
 *
 */

@Service(name="resume-domain")
@ExecuteOn(RuntimeType.DAS)  // Only appropriate for the DAS
@Scoped(PerLookup.class)
@CommandLock(CommandLock.LockType.NONE)
@I18n("quiesce.command.resume")
public class ResumeDomain implements AdminCommand {

    @Inject
    private AdminCommandLock adminCommandLock;

    @Inject
    Logger logger;

    private static final LocalStringsImpl strings =
            new LocalStringsImpl(ResumeDomain.class);


    public void execute(AdminCommandContext context) {

        ActionReport report = context.getActionReport();

        Thread commandLockThread = adminCommandLock.resumeCommands();

        if (commandLockThread == null) {
            report.setMessage(strings.get("quiesce.notsuspended"));
            report.setActionExitCode(ActionReport.ExitCode.FAILURE);
            return;
        }

        /*
         * We don't want resume-domain to return until the
         * suspendedDomainLockThread has completed its work.  We wait
         * until the thread is no longer alive.
         */
        while (commandLockThread.isAlive()) {
            try {
                commandLockThread.join();
            } catch (java.lang.InterruptedException e) {
            }
        }

        String DATE_FORMAT = "EEE, dd MMM yyyy HH:mm:ss z";
        SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT);
        logger.log(Level.INFO, strings.get("quiesce.resumesuccessful",
                                           sdf.format(new Date())));
        report.setMessage(strings.get("quiesce.resumesuccessful",
                                           sdf.format(new Date())));
        report.setActionExitCode(ActionReport.ExitCode.SUCCESS);
        return;
    }
}
